-----------------------------------------------------------------------------------------------
-- Client Lua Script for SSDps
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- SSDps Module Definition
-----------------------------------------------------------------------------------------------
local SSDps = {} 


-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function SSDps:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables
	self.Activated = true
	self.svWindowLocation = nil
	self.SpellSurgeShowing = true
	self.IgniteShowing = 0
	self.Opacity = 0.75
	self:Reset()
	
	return o
end


function SSDps:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "SSDps"
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end


function SSDps:Reset()
	self.FirstDmgTicks = 0
	self.LastDmgTicks = 0
	self.TotalDmg = 0
	self.LastTotalDmg = 0
	self.LastTextUpdate = 0
end


-----------------------------------------------------------------------------------------------
-- SSDps OnLoad
-----------------------------------------------------------------------------------------------
function SSDps:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("SSDps.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- SSDps OnDocLoaded
-----------------------------------------------------------------------------------------------
function SSDps:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "SSDpsForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		--Set up the text boxes to output to
		self.tAverageDPS = self.wndMain:FindChild("AverageDPS")
		self.tTotalDMG = self.wndMain:FindChild("TotalDMG")
		self.tBurstDPS = self.wndMain:FindChild("BurstDPS")
		self.tTitle = self.wndMain:FindChild("Title")
		self.SpellSurge = self.wndMain:FindChild("SpellSurge")
		self.Ignite = self.wndMain:FindChild("Ignite")
		
		--And the timer that monitors for Spellsurge and updates the text
		self.Timer = ApolloTimer.Create(0.25, true, "OnTimer", self)
		
		self.Ignite:SetOpacity(0, 0)
		self.wndMain:SetOpacity(self.Opacity, 0)
		
		--Move window to saved window position
		if self.svWindowLocation then
			self.wndMain:MoveToLocation(self.svWindowLocation)
		end
		
		if self.Activated then
		    self.wndMain:Show(true, true)

		else 
			self.Timer:Stop()
			self.wndMain:Close()
		end
		
		-- Register event handlers
		Apollo.RegisterSlashCommand("ssdps", "OnSSDpsOn", self)
		Apollo.RegisterEventHandler("CombatLogDamage", "OnCombatLogDamage", self)
	end
end

-----------------------------------------------------------------------------------------------
-- SSDps Functions
-----------------------------------------------------------------------------------------------


function SSDps:OnConfigure()
	self.wndConfig = Apollo.LoadForm(self.xmlDoc, "SSDpsConfig", nil, self)

	local slider = self.wndConfig:FindChild("SliderBar")
	slider:SetValue(self.Opacity * 100)
	self.wndConfig:Show(true, true)
end


function SSDps:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return nil
	end
	local wLoc = self.wndMain:GetLocation()
	local tSave = 
	{
		winLoc = wLoc:ToTable(),
		activated = self.Activated,
		version = 4
	}
	tSave.activated = self.Activated

	return tSave
end

function SSDps:OnRestore(eType, tSave)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return nil
	end
		
	if tSave.version then
		if tSave.version >= 3 then
			if tSave.activated then
				self.Activated = true
			else 
				self.Activated = false
			end	
		end

		if tSave.version >= 4 then  --Ignore old window size
			if tSave.winLoc then
				self.svWindowLocation = WindowLocation.new(tSave.winLoc)
				if self.wndMain then
					self.wndMain:MoveToLocation(self.svWindowLocation)
				end	
			end
		end
	end
end


-- on SlashCommand "/ssdps"
function SSDps:OnSSDpsOn()
	if self.Activated then
		self.Timer:Stop()
		self.wndMain:Close()
		self:Reset()
		self.Activated = false
	else
		self.Timer:Start()
		self.wndMain:Invoke()
		self.Activated = true
	end
end

-- Set the text of the stat windows
function SSDps:SetText(total, burst, avg)
	self.tTotalDMG:SetText(string.format("%d", total))
	self.tAverageDPS:SetText(string.format("%d", avg))
	self.tBurstDPS:SetText(string.format("%d", burst))
end

-- Run quickly for Spell Surge, and if we're in combat do extra stuff
function SSDps:OnTimer()
	if GameLib.IsSpellSurgeActive() then
		if not self.SpellSurgeShowing then
			self.SpellSurge:Show(true, true)
			self.SpellSurgeShowing = true
		end
	else
		if self.SpellSurgeShowing then
			self.SpellSurge:Show(false, true)
			self.SpellSurgeShowing = false
		end
	end
	
	if self.IgniteShowing == 1 then
		self.IgniteShowing = 2
		self.Ignite:SetOpacity(1, 1000)
	elseif self.IgniteShowing == 2 then
		self.IgniteShowing = 0
		self.Ignite:SetOpacity(0, 0)
	end
	
	if self.LastDmgTicks == 0 then return end
	
	local now = Apollo.GetTickCount()
	local updateDiff = now - self.LastTextUpdate
	
	if now - self.LastDmgTicks > 6000 then --Fight's over		
		local fightTicks = self.LastDmgTicks - self.FirstDmgTicks
		local fightDPS = 1000 * self.TotalDmg / fightTicks
		if fightTicks < 1000 then fightDPS = self.TotalDmg end
		self:SetText(self.TotalDmg, 0, fightDPS)
		self.Timer:Set(0.25, true, "OnTimer", self)
		self:Reset()
		self.tTitle:SetText(string.format("Last Fight: %0.1f seconds", fightTicks / 1000))
		return
	elseif updateDiff > 970 then --Trying to update every second. 970ms accounts for possible timer lag
		local combatTime = (now - self.FirstDmgTicks) / 1000
		local dps = 1000 * self.TotalDmg / (now - self.FirstDmgTicks)
		local burst = 1000 * (self.TotalDmg - self.LastTotalDmg) / updateDiff
		self:SetText(self.TotalDmg, burst, dps)
		self.tTitle:SetText(string.format("Combat: %0.1f seconds", combatTime))
		self.LastTotalDmg = self.TotalDmg
		self.LastTextUpdate = now
	end
end


function SSDps:OnCombatLogDamage(evt)
	if not self.Activated then return end
	--bail out immediately if damage not done by this player
	if not (evt and evt.unitCaster == GameLib.GetPlayerUnit()) then	return end
	
	local now = Apollo.GetTickCount()
		
	if self.FirstDmgTicks == 0 then
		self.FirstDmgTicks = now
		self.LastTextUpdate = now
		self.tTitle:SetText("Combat started.")
		self.Timer:Set(0.1, true, "OnTimer", self)
	end
	
	if evt.splCallingSpell then
		if evt.splCallingSpell:GetName() == "Ignite" and evt.bPeriodic == false then
			self.IgniteShowing = 1
		end
	end
		
	self.TotalDmg = self.TotalDmg + evt.nDamageAmount
	self.LastDmgTicks = now
end

---------------------------------------------------------------------------------------------------
-- SSDpsConfig Functions
---------------------------------------------------------------------------------------------------

function SSDps:OnConfigSliderChanged( wndHandler, wndControl, fNewValue, fOldValue )
	if (fNewValue > 10 and fNewValue <= 100) then
		self.Opacity = fNewValue / 100.0;
		self.wndMain:SetOpacity(self.Opacity, 0)
	end
end

function SSDps:OnCloseButton( wndHandler, wndControl, eMouseButton )
	self.wndConfig:Close()
end

-----------------------------------------------------------------------------------------------
-- SSDps Instance
-----------------------------------------------------------------------------------------------
local SSDpsInst = SSDps:new()
SSDpsInst:Init()
